/* 

    This program deletes tuples from the database at periodic intervals. The
    program takes four command line arguments 

    (1) the periodicity with which the daemon will be invoked.
    (2) let 'x' be the difference of the current time and the timestamp of a
        tuple in seconds. let 'y' be the second command line argument.
	whenever 'x' exceeds 'y', the tuple is deleted from the database.
    (3) the name of the relation from which the tuples are to be deleted.
    (4) the name of the timestamp attribute in that relation in the database.

    This can be used with any relation where we want to delete all the tuples
    based on one field being less than some certain value. Also the periodicity
    with which this deletion is to be carried out can also be specified as the 
    command line argument as stated above.

    The validity of the sign and the format of the period and the expiry time 
    is checked.

*/

import java.sql.*;
import java.io.*;
import java.util.*;
import java.net.*;

class DeleteDaemon {

    public static long period, expiry, start;

    public static void main (String args[])
    throws InterruptedException
    {
        Thread t = Thread.currentThread();
	try {
	    period = Integer.parseInt(args[0]);
	    expiry = Integer.parseInt(args[1]);
	// check that expiry and period are positive 
	    if ((period <= 0) || (expiry <= 0)) {
	        throw new NumberFormatException();
	    }
	}
	catch (NumberFormatException e) {
	    System.err.println ("Wrong format of expiry or period. Both are" +
	    	" required to be greater than zero");
	    System.exit(1);
	}
	catch (ArrayIndexOutOfBoundsException e) {
	    System.err.println
	    	("Usage : java DeleteDaemon period expiry rel_name attribute");
	    System.exit(1);
	}
	Thread.sleep(4000);   // generally sleep for some time initially
	long end, diff;
        while (true) {
	    start = System.currentTimeMillis();
	    try {
    	        deleteTuples(args[2],args[3]);
	    }
	    catch (ArrayIndexOutOfBoundsException e) {
	        System.err.println
	    	("Usage : java DeleteDaemon period expiry rel_name attribute");
	        System.exit(1);
	    }
	    end = System.currentTimeMillis();
    // diff = time taken to delete the tuples. note that 'end' and 'start' are 
    // in milliseconds and 'period' is specified in seconds
	    diff = (end - start)/1000; 
	    if (diff < period) {
	        Thread.sleep((period - diff)*1000);
	    }
	}
    }

    public static void deleteTuples (String rel_name, String attribute)
    {
    	String username = "ashish";
	String password = "ashish";
        try 
	{
    // auto commit has been set to false, so an explicit commit stmt is needed
           DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
	    Connection con = DriverManager.getConnection
	        ("jdbc:oracle:thin:@:1521:GEN",username,password);
	    con.setAutoCommit (false); 
	    Statement stmt = con.createStatement();
	   
    // note that expiry is in 'seconds' and start is in 'milliseconds'
    // and the database stores the time in 'seconds'.
	    stmt.executeQuery ("DELETE FROM " + rel_name + " WHERE " +
	        attribute + " < " + ((start/1000) - expiry));

	    con.commit();  // commit only after updating the offset
	}
	catch (ArrayIndexOutOfBoundsException e) {
	    System.err.println 
	    	("Usage : java DeleteDaemon period expiry rel_name attribute");
	    System.exit (1);
	}
	catch (Exception e)
	{
	    System.err.println ("Improper column name or relation name");
	    System.exit (1);
	}
    }
}
