/*

    The program takes two command line arguments 
    	[rawlogfile] : name of the raw log file
	[period]     : period (in seconds) after which the database is to be
		       updated.
    In case wrong number of arguments are supplied, it points out the error 
    and exits. Also in case there is any error in the format of the arguments,
    it will again exit with appropriate message.

    The program assumes that the file name which is passed to this Daemon as
    command line argument exists. It does no check what so ever about the
    existence of the file or the read permissions of the file.

    The Daemon invokes a perl script which writes its output to two files,
    namely 
    	'/tmp/abc0' and '/tmp/abc1'
    from which the Daemon reads the values to be inserted into the database. 
    The description of these two files can be read in the documentation of the
    perl script. 
    
*/
import java.sql.*;
import java.io.*;
import java.util.*;

class ProxyMainDaemon {

    private static long prev_offset;
    private static String prev_time;
    private static long period;
    private static String query = 
    	new String("INSERT INTO "+"proxy_log"+" VALUES (");
    private static String temp1, temp;

    public static void main (String args[])
    throws InterruptedException
    {
        Thread t = Thread.currentThread();
    // sleep for some time initially
	Thread.sleep(3000);	
	long start, end, diff;
        while (true) {
	    start = System.currentTimeMillis();
	    try {
		period = Integer.parseInt(args[1]);
    // period must be a positive integer
		if (period <= 0) {
		    throw new NumberFormatException();
		}
    // pass rawlogfile name as parameter to the procedure that inserts tuples
    	        insert_tuples(args[0]);	   
	    }
	    catch (ArrayIndexOutOfBoundsException e) {
		System.err.println
		    ("Usage:java ProxyMainDaemon [rawlogfile] [period]");
		System.exit(1);
    	    }
	    catch (NumberFormatException e) {
		System.err.println
		    ("Usage:java ProxyMainDaemon [rawlogfile] [period]");
		System.err.println("[period] must be greater than zero");
		System.exit(1);
	    }
	    end = System.currentTimeMillis();
    // note that end and start are in milliseconds, where as the period is
    // specified in seconds. Thus we divide by 1000.
	    diff = (end - start)/1000;
    // go into sleep only if the time taken to insert the current batch is less
    // than the period of update. If update time >= period, then no sleep.
    // straight away go into the next batch update.
	    if (diff < period) {
	        long temp = period - diff;
    // the argument taken by sleep is in milliseconds.
	        Thread.sleep(temp*1000); 
	    }
	}
    }

/*
    An important point to take note is that the daemon commits all the 
    insertions at the end after the new values of the offsets have been written
    to the database. This is to ensure that no tuple gets duplicated in the
    database, in case the daemon gets interrupted in the middle of the 
    insertion process.
*/

    public static void insert_tuples (String rawlog)
    {
    	String username = "ashish";
	String password = "ashish";
        try 
	{
    // auto commit has been set to false, so an explicit commit stmt is needed
           DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
	    Connection con = DriverManager.getConnection
	        ("jdbc:oracle:thin:@:1521:GEN",username,password);
	    con.setAutoCommit (false); 
	    Statement stmt = con.createStatement();
	   
	    ResultSet rs = stmt.executeQuery
	        ("SELECT * FROM proxy_offset ");
	    rs.next();

    // get the previous values of the offsets
	    prev_offset = rs.getInt("prev_offset");
	    prev_time = rs.getString("prev_time");

    // run the perl script proxy to process the raw logfile proxy_log
	    Runtime r = Runtime.getRuntime();
	    Process p = null;
	    p = r.exec ("proxy.perl "+rawlog+" /tmp/abc0 /tmp/abc1 "+ 
	        prev_offset + " " + prev_time);
	    p.waitFor();
	    
    // open a stream for input from file
	    FileInputStream infile = new FileInputStream ("/tmp/abc1");
	    InputStreamReader reader = new InputStreamReader (infile);
	    BufferedReader input = new BufferedReader (reader);

    // read from the input file, the new values of offsets
	    temp = input.readLine();
	    prev_offset = Integer.parseInt(temp);
	    prev_time = input.readLine();

	    infile.close();
	    reader.close();
	    input.close();
	    
	    infile = new FileInputStream ("/tmp/abc0");
	    reader = new InputStreamReader (infile);
	    input = new BufferedReader (reader);

	    temp = input.readLine();
	    while (temp != null)
	    {
		try {
	            temp1 = query + temp + ", ";
	            temp = input.readLine();
		    rs = stmt.executeQuery("SELECT ip_address FROM ip_machine "+
		        " WHERE machine_name = '" + temp + "'");
    /* 
    	check if the entry already exists in the database. If it exists, then 
	get the corresponding ip number and use it to insert the log file entry
	in to the database. If the entry does not exist, then use the function
	ipAddress() to get the ipno for that machine. Insert this new value
	into the table which stores the mapping between the ip_nos and the
	machine names. and also use this to insert the new log file entry into
	the database.
    */
		    if (rs.next()) { 
		    	temp1 = temp1+"'" + rs.getString("ip_address") + "')\n";
		    }
		    else {
		    	String ip_no = IPAddress.ipAddress (temp);
		    	stmt.executeUpdate ("INSERT INTO ip_machine VALUES ('" +
		            ip_no + "', '" + temp +"')");
		    	temp1 = temp1 + "'" + ip_no + "')\n";
		    }
		    stmt.executeUpdate(temp1);
		}
		catch (Exception e) {
		    // do nothing
		}
	        temp = input.readLine(); 
	    }
    // update the values of offsets in the database
	    stmt.executeUpdate 
	    ("UPDATE proxy_offset set prev_offset = " + prev_offset);
	    stmt.executeUpdate 
	    ("UPDATE proxy_offset set prev_time = " + prev_time);
    // commit only after updating the offset
	    con.commit(); 
	    infile.close();
	    reader.close();
	    input.close();
	}
	catch (Exception e)
	{
	    System.out.println (e);
	}
    }
}
